#!/usr/bin/python3.4

####### IMPORT SECTION #################################################

#Basic modules
import csv
import matplotlib.pyplot as plt
import math
import numpy as np
#Image module
import scipy.misc as misc 
from scipy.misc import imread
from scipy.misc import imsave
#Abel module
import abel as abel
import abel.transform as transform
from abel.transform import Transform 
#Integration module
import scipy.integrate as integ 
from scipy.integrate import trapz
#For a nmber of files in a directory
import os, os.path
#For finding the index of the pxel charactricterized by the highest value of intensity
from numpy import unravel_index

# import calibration list for every wavelength from Calibration script
from Calibration import cor_fac

#Data:
rho_CH4 = 0.668 #kgN/m3N
rho_CO2 = 1.842 #kgN/m3N
rho_O2 = 1.331 #kgN/m3N
rho_N2 = 1.165 #kgN/m3N
M_CH4 = 16 #kg/kmol
M_CO2 = 44 #kg/kmol
M_O2 = 32 #kg/kmol
M_N2 = 28 #kg/kmol

CH4_flow = np.array([0.7, 1.3, 2.2, 3.6, 5.8, 7.1]) #Nl/min fuel and oxidizer flow is variable
ox_flow = np.array([4.2, 6.6, 10.0, 14.3, 21.1, 23.6]) #Nl/min oxidizer flow
#diameters:
r_CH4 = 0.001 #m
r_ox = 0.025 #m
# mass flows:
CH4_flow_mass_array = []
ox_flow_mass_array = []
#oxidizer content 
ox_cont_array =  []
# SIGNALS
OH_signal_array = []
CH_signal_array = []
CO2_signal_array = []
#mole fractions:
n_frCH4_array = []
n_frO2_array = []
n_frN2_array = []
n_frCO2_array = []
#Mass flows of fuel and oxidizer:
CH4_flow_mass_array = []
ox_flow_mass_array = []

#choose CO2 or N2 (air) as a diluent of oxygen
Marker = 'N2'

#file directory
if Marker == 'N2':
	file_directory = '/home/jb0183/Measurements/Results/Premixed_N2/'
if Marker == 'CO2':
	file_directory = '/home/jb0183/Measurements/Results/Premixed_CO2/NEW/'

number_of_files = len([name for name in os.listdir(file_directory) if os.path.isfile(os.path.join(file_directory, name))])

My_range = number_of_files - 1 #minus 1 file with other data
ox_start = 35 #starting value of the oxidizer content
for i in range(My_range):
	ox_cont = ox_start + 5*i 
	ox_cont_array.append(ox_cont)
	print(ox_cont)
	
	if Marker == 'N2':
		file_name = '1premixed-N2-' + str(ox_cont) + '_1'
	if Marker == 'CO2':
		file_name = 'KGpremixed' + str(ox_cont) + '_1'
	
	#mass flow of CH4

	CH4flow_mass = CH4_flow[i] * (1/60) * (1/1000) * rho_CH4 #(kg/s)
	CH4_flow_mass_array.append(CH4flow_mass)
	
	#mass flow of O2 
	O2_flow = (ox_cont/100) * ox_flow[i]
	O2flow_mass = O2_flow * (1/60) * (1/1000) * rho_O2 #(kg/s)
	
	if Marker == 'N2':
		#mass flow of N2 
		
		N2_flow = (1-(ox_cont/100)) * ox_flow[i]
		N2flow_mass = N2_flow * (1/60) * (1/1000) * rho_N2 #(kg/s)
		
		#mass fractions
		mfrCH4 = CH4flow_mass/(O2flow_mass + N2flow_mass + CH4flow_mass)
		mfrO2 = O2flow_mass/(O2flow_mass + N2flow_mass + CH4flow_mass)
		mfrN2 = N2flow_mass/(O2flow_mass + N2flow_mass + CH4flow_mass)
			
		#mole fraction 
		nfrCH4 = (mfrCH4/M_CH4) / (mfrO2/M_O2 + mfrN2/M_N2 + mfrCH4/M_CH4)
		nfrO2 = (mfrO2/M_O2) / (mfrO2/M_O2 + mfrN2/M_N2 + mfrCH4/M_CH4)
		nrfN2 = (mfrN2/M_N2) / (mfrO2/M_O2 + mfrN2/M_N2 + mfrCH4/M_CH4)
		n_frCH4_array.append(nfrCH4)
		n_frO2_array.append(nfrO2)
		n_frN2_array.append(nrfN2)
		
		########################
		
		#density of oxidizer:
		rho_ox	= (ox_cont/100) * rho_O2 + (1 - (ox_cont/100)) * rho_N2	
		#mass flow of oxidizer:
		ox_flow_mass = ox_flow[i] * (1/60) * (1/1000) * rho_ox * 1/(3.14*(math.pow(r_ox,2))) #(kg/m^2/s)
		ox_flow_mass_array.append(ox_flow_mass)
	
	if Marker == 'CO2':
		#mass flow of CO2 
		CO2_flow = (1-(ox_cont/100)) * ox_flow[i]
		CO2flow_mass = CO2_flow * (1/60) * (1/1000) * rho_CO2 #(kgN/s)
		
		#mass fraction
		mfrCH4 = CH4flow_mass/(O2flow_mass + CO2flow_mass + CH4flow_mass)
		mfrO2 = O2flow_mass/(O2flow_mass + CO2flow_mass + CH4flow_mass)
		mfrCO2 = CO2flow_mass/(O2flow_mass + CO2flow_mass + CH4flow_mass)
			
		#mole fraction 
		n_frCH4 = (mfrCH4/M_CH4) / (mfrO2/M_O2 + mfrCO2/M_CO2 + mfrCH4/M_CH4)
		n_frO2 = (mfrO2/M_O2) / (mfrO2/M_O2 + mfrCO2/M_CO2 + mfrCH4/M_CH4)
		n_rfCO2 = (mfrCO2/M_CO2) / (mfrO2/M_O2 + mfrCO2/M_CO2 + mfrCH4/M_CH4)
		n_frCH4_array.append(n_frCH4)
		n_frO2_array.append(n_frO2)
		n_frCO2_array.append(n_rfCO2)
			
		########################
		
		#density of oxidizer:
		rho_ox	= (ox_cont/100) * rho_O2 + (1 - (ox_cont/100)) * rho_CO2
		#mass flow of oxidizer:
		ox_flow_mass = ox_flow * (1/60) * (1/1000) * rho_ox * 1/(3.14*(math.pow(r_ox,2))) #(kg/m^2/s)
		ox_flow_mass_array.append(ox_flow_mass)

	####### USER'S DATA SECTION ############################################

	if Marker == 'CO2':
		#define image size (physical dimension) - number of pixels should be odd 
		Left = 74
		Right = 171
	
	if Marker == 'N2':
		#define image size (physical dimension) - number of pixels should be odd 
		Left = 70
		Right = 181 + 20

	####### READ DATA FILE SECTION #########################################
	
	full_directory = file_directory + file_name + '.txt'
	file1 = open(full_directory,'r')

	#read all the lines
	all_lines = []
	with file1 as f:
		for line in f:
			inner_list = [elt.strip() for elt in line.split(',')]
			all_lines.append(inner_list)

	#delete empty space in the first line (it must be done - in other case an error occurs)
	del all_lines[0][-1] 
	#delete the first element in every list
	loops = len(all_lines)
	for i in range(loops):
		del all_lines[i][0] 
		
	#convert list of strings to list of floats
	all_lines = [[float(float(j)) for j in i] for i in all_lines]

	#CLOSE TXT FILE !
	file1.close()

	######### PREPARE X VALUES ############################################

	# creat a list of values for X axis (wavelength) - to cut off ,,dead area''
	cut_L = 30
	cut_R = 980
	x1 = all_lines[1][cut_L:cut_R]

	####### IMAGE (MATRIX) PROCESSING SECTION (1/2) #######################
	############ Image 1 ############ 
	MATRIX1 = all_lines[Left:Right]
	print(i)
	imsave('111'+str(ox_cont)+'.png',MATRIX1)
	############ Image 2 ############ 
	# 90 Degree Clockwise Rotation 
	MATRIX2 = list(zip(*MATRIX1))
	#Convert a list of tuples to a list of lists
	MATRIX2 = [list(elem) for elem in MATRIX2]
	imsave('222.png',MATRIX2)
	######### ABEL TRANSFORMATION ##########################################
	# Image for Abel transformation
	IM = imread('222.png')
	rows, cols = IM.shape    # image size
	# Image center-line should be mid-pixel, i.e. odd number of columns
	if cols % 2 == 0:
		print ("HL: even pixel width image, re-adjusting image centre using `slice` method, returning odd-width size image")
		IM = abel.tools.center.center_image(IM, center="slice", odd_size=True)
		rows, cols = IM.shape   # new image size
	# dr=0.5 may help reduce pixel grid coarseness
	# NB remember to also pass to angular_integration
	AIM = abel.Transform(IM, method='onion_bordas',
						 use_quadrants=(True, True, True, True),
						 symmetry_axis=None,
						 verbose=True).transform					 
	####### IMAGE (MATRIX) PROCESSING SECTION (2/2) ########################
	############ Image 3 ############ ; after Abel transformation 
	imsave('333.png',AIM)
	############ Image 4 ############ 
	# 90 Degree CounterClockwise Rotation 
	IM_a = list(zip(*list(AIM)))
	#Convert a list of tuples to a list of lists
	IM_a = [list(elem) for elem in IM_a]
	imsave('144.png',IM_a)

	############ INTENSITY CHART  ############ 
	# choose an image IM_a - after Abel  MATRIX1 = before Abel
	Choose_Image = MATRIX1
	
	if Marker == 'CO2':
		####################### OH #######################
		#summing pixel arrays 
		y_up_OH = 8
		y_down_OH = 30
		# prepare intensity values (taking into account calibration by correction factor)
		y1_OH = cor_fac * np.array((np.sum(Choose_Image[y_up_OH:y_down_OH], axis = 0)[cut_L:cut_R]))
		####################### CH #######################
		#summing pixel arrays 
		y_up_CH = 17
		y_down_CH = 39
		y1_CH = cor_fac * np.array((np.sum(Choose_Image[y_up_CH:y_down_CH], axis = 0)[cut_L:cut_R]))
			####################### CO2 #######################
		y1_CO2 = y1_CH 
		
	if Marker == 'N2':
		####################### OH #######################
		#summing pixel arrays 
		y_up_OH = 14
		y_down_OH = 35
		# prepare intensity values (taking into account calibration by correction factor)
		y1_OH = cor_fac * np.array((np.sum(Choose_Image[y_up_OH:y_down_OH], axis = 0)[cut_L:cut_R]))
		####################### CH #######################
		#summing pixel arrays 
		y_up_CH = 24
		y_down_CH = 42
		y1_CH = cor_fac * np.array((np.sum(Choose_Image[y_up_CH:y_down_CH], axis = 0)[cut_L:cut_R]))
		####################### CO2 #######################
		y1_CO2 = y1_CH 
		
	############ Intensity charts ############ 
	
	### limits for Intensity and integration charts ###
	X_lim_L = 250
	X_lim_R = 550
	

	fig1 = plt.figure(1)
	'''
	###  OH signal ###
	plot1 = fig1.add_subplot(3,1,1) 
	plot1.plot(x1,y1_OH, label = str(ox_cont))
	plt.title('OH signal', fontsize=12, fontweight='bold')
	plt.xlabel('wavelength (nm)')
	plt.ylabel('Intensity')
	plt.ylim([0,50000])
	plt.xlim([X_lim_L,X_lim_R])
	plt.legend()
	### CH and CO2 singal ###
	plot2 = fig1.add_subplot(3,1,2) 
	plot2.plot(x1,y1_CH,label = str(ox_cont))
	plt.title('CH and CO2 signal', fontsize=12, fontweight='bold')
	plt.xlabel('wavelength (nm)')
	plt.ylabel('Intensity')
	plt.ylim([0,50000])
	plt.xlim([X_lim_L,X_lim_R])
	plt.legend()
	'''
	####### final chart - half from OH range, half from CH and CO2 range  ###########
	
	y1_final = list( y1_OH[:(len(y1_OH)/2)] ) + list( y1_CH[(len(y1_CH)/2):] )
	y1_final = [x/1000 for x in y1_final]

	
	plot3 = fig1.add_subplot(1,1,1) 
	plot3.plot(x1,y1_final,label = 'X O2 = ' + str(ox_cont) + '%')
	#plt.title('CH and CO2 signal', fontsize=12, fontweight='bold')
	plt.xlabel('Wavelength (nm)')
	plt.ylabel('Intensity')
	plt.ylim([0,(max(y1_final) + 0.12 * max(y1_final))])
	plt.xlim([X_lim_L,X_lim_R])
	plt.legend()
	plt.savefig('Complete_signal' + Marker + '.jpeg')
	
	####### SAVE TO CSV ################

	data1 = open('Intensity as a function' + str(ox_cont) + 'of a wavelength.csv','w') 
	csvwriter1 = csv.writer(data1) 
	csvwriter1.writerow(x1)
	csvwriter1.writerow(y1_final)
	data1.close()

	####### INTEGRATION FUNCTION ###################################################
	def Integration (int_L,int_R,y1):
		
		#integrated part of the signal
		x1_int = all_lines[1][int_L:int_R]
		y1_int = y1[int_L:int_R]
		
		#integration of the signal with background
		int_all = trapz(np.array(y1_int),np.array(x1_int))
	
		############## trend line to cut off the bottom ######################
		#range as a basis for trend line
		LeftL = int_L - 70
		LeftR = int_L # left side of integration range
		RighrL = int_R # right side of integration range
		RightR = int_R + 70

		x1_base_tr = np.array(all_lines[1][LeftL:LeftR] + all_lines[1][RighrL:RightR])
		y1_base_tr = np.array(list(y1[LeftL:LeftR]) + list(y1[RighrL:RightR]))

		#coefficients for polynomial, 4th degree
		coe_tr = np.polyfit(x1_base_tr,y1_base_tr,6)

		#create y_cutoff values for model according to coefficients
		loops = len(x1_int)
		y_cutoff = []
		for i in range(loops):
			y_fit_model = coe_tr[0]*math.pow(x1_int[i],6) + coe_tr[1]*math.pow(x1_int[i],5) + coe_tr[2]*math.pow(x1_int[i],4) + coe_tr[3]*math.pow(x1_int[i],3) + coe_tr[4]*math.pow(x1_int[i],2) + coe_tr[5]*x1_int[i] + coe_tr[6]
			y_cutoff.append(y_fit_model)

		# integration with cutted area 
		int_cutoff = trapz(np.array(y_cutoff),np.array(x1_int))

		#signal - area of total signal minus background
		signal = int_all - int_cutoff

		return(signal, x1_int, y1_int, x1_base_tr, y1_base_tr, y_cutoff, int_all)
	
	####### OH  ###################################################
	#range  - the range is moved to the right by ,,cut_L'' value
	OH_L = 145 - cut_L
	OH_R = 220 - cut_L
	OH_data = Integration(OH_L,OH_R,y1_OH)
	OH_signal_array.append(OH_data[0])
	print('OH  signal',OH_data[0])
	
	####### CH  ###################################################
	CH_L = 590 - cut_L
	CH_R = 660 - cut_L
	CH_data = Integration(CH_L,CH_R,y1_CH)
	CH_signal_array.append(CH_data[0])
	print('CH  signal',CH_data[0])
	
	####### CO2  ###################################################

	CO2_L = 540 - cut_L
	CO2_R = 580 - cut_L
	CO2_data = Integration(CO2_L,CO2_R,y1_CO2)
	CO2_signal_array.append(CO2_data[6])
	print('CO2  signal',CO2_data[6])
	
	#INTEGRATION CHECK
	
	### OH ### 
	fig100 = plt.figure(100)
	plot1 = fig100.add_subplot(1,1,1) 
	plot1.plot(OH_data[1],OH_data[2])
	plot2 = fig100.add_subplot(1,1,1) 
	plot2.plot(OH_data[3],OH_data[4])
	plot3 = fig100.add_subplot(1,1,1) 
	plot3.plot(OH_data[1],OH_data[5])
	
	### CH ### 
	plot4 = fig100.add_subplot(1,1,1) 
	plot4.plot(CH_data[1],CH_data[2])
	plot5 = fig100.add_subplot(1,1,1) 
	plot5.plot(CH_data[3],CH_data[4])
	plot6 = fig100.add_subplot(1,1,1) 
	plot6.plot(CH_data[1],CH_data[5])
	
	### CO2 ### 
	plot7 = fig100.add_subplot(1,1,1) 
	plot7.plot(CO2_data[1],CO2_data[2])
	
	plt.xlabel('Wavelength (nm)')
	plt.ylabel('Intensity')
	plt.ylim([0,(max(OH_data[2]) + 0.12 * max(OH_data[2]))])
	plt.xlim([X_lim_L,X_lim_R])
	
	#plt.show()

xlim = ( min(ox_cont_array) - 1 ),( max(ox_cont_array) + 1 )

fig3 = plt.figure(3)
plot1 = fig3.add_subplot(3,1,1) 
plot1.scatter(ox_cont_array[0:My_range],OH_signal_array)
plt.xlabel('Oxygen content in oxidizer')
plt.ylabel('OH signal')
plt.xlim(xlim)
plt.ylim([0,(max(OH_signal_array) + 0.1 * max(OH_signal_array))])
plot2 = fig3.add_subplot(3,1,2) 
plot2.scatter(ox_cont_array[0:My_range],CH_signal_array)
plt.xlabel('Oxygen content in oxidizer')
plt.ylabel('CH signal')
plt.xlim(xlim)
plt.ylim([0,(max(CH_signal_array) + 0.1 * max(CH_signal_array))])
plot3 = fig3.add_subplot(3,1,3) 
plot3.scatter(ox_cont_array[0:My_range],CO2_signal_array)
plt.xlabel('Oxygen content in oxidizer')
plt.ylabel('CO2 signal')
plt.xlim(xlim)
plt.ylim([0,(max(CO2_signal_array) + 0.1 * max(CO2_signal_array))])
plt.savefig('MEASUREMENTS_' + Marker + '.jpeg')

plt.show()

# change all the values
#x1 = [x+15 for x in x1]

