#!/usr/bin/python3.4

import cantera as ct
import csv
import matplotlib.pyplot as plt
import math

# define a mixture 
mixture = ct.Solution('gri30MOD.cti')

# a file in wchich the results will be written
myfile = open('equilibration_results.csv','w')
csvwriter = csv.writer(myfile)
# labels
csvwriter.writerow(['Tb, K','Ta, K','X H3O+','Y H3O+','X e-','Y e-',
					'n_e, 1/cm3','n_e, mole/gram','\sigma_E, S/m',
					'\sigma_H3O+, S/m', '\sigma_all, S/m'])

# initial temperaure 
T = 250
dT = 50
# temperature
x1 = []
y1 = []

# the loop for calculating all of the relevant parameters 
# for different initial temperatures
while T <= 1900:
	
	# define mixture parameters
	mixture.TP = T, p, 
	mixture.X = 'CH4:1, O2:2'
	
	# initial temperature of the mixture
	Tb = mixture.T
	
	# mole fractions of the mixture
	mole = mixture.X
	
	# create the reactor, and fill it with mixture
	reactor = ct.IdealGasConstPressureReactor(mixture)
	
	# H3O+ and e- mole and mass fractions within the reactor
	XH3O_b = reactor.thermo['H3O+'].X
	YH3O_b = reactor.thermo['H3O+'].Y
	print('H3O+ :','X :', XH3O_b,'Y :', YH3O_b)
	Xe_b = reactor.thermo['e-'].X
	Ye_b = reactor.thermo['e-'].Y
	print('e- :','X :', Xe_b,'Y :', Ye_b)

	# equilibriation with constant enthalpy and pressure
	mixture.equilibrate('HP')
	
	# mole fractions of the mixture 
	mole = mixture.X
	print(mole)
	print(mixture())
	
	# create the reactor, and fill it with mixture 
	reactor = ct.IdealGasConstPressureReactor(mixture)
	
	# H3O+ and e- mole and mass fractions of the reactor 
	XH3O_a = reactor.thermo['H3O+'].X
	YH3O_a = reactor.thermo['H3O+'].Y
	print('H3O+ :','X :', XH3O_a,'Y :', YH3O_a)
	Xe_a = reactor.thermo['e-'].X
	Ye_a = reactor.thermo['e-'].Y
	print('e- :', 'X :',Xe_a, 'Y :',  Ye_a)
	
	# CALCULATION OF ELECTRICAL CONDUCTIVITY:
	Cg = mixture.density / mixture.mean_molecular_weight #(kmol/m3)

	# Avagadro Constant 
	No = 6.022E26 			#(1/kmol)
	# all particles concentration
	N = Cg * No				#(1/m3)
	
	# ELECTRONS
	# data:
	e_e = 1.60217662E-19 		#(C = A*s) 	elementary charge of electron
	m_e = 9.109E-31 			#(kg)		mass of one electron
	vN_e = 17.6E-8 * 1/1000000	#(m3/s)		effective collision frequency
								#			of electrons in atmospheric gases
								#			per N 
							
	# effective collision frequency of electrons in atmospheric gases
	v_e = vN_e * N				#(1/s) or (particles / s)
	
	# electron concentration
	n_e = N * Xe_a				#(1/m3)
	print('electron concentration:',n_e/1000000,'1/cm3')
	
	n_e_mole = n_e / (6.022E23 * mixture.density * 1000) #(mole/gram)
	print('electron concentratuon:',n_e_mole,'mole/gram')
	
	#electrical conductivity of electrons
	sigma1 = ( e_e*e_e * n_e ) / ( m_e * v_e )
	print('electrical conductivity E:',sigma1, '(A2 * s3)/(kg * m3)')
	
	# IONS
	# data:
	e_i = 1.60217662E-19 		#(C = A*s) 	elementary charge of electron
	m_i = 3.159E-26 			#(kg)		mass of the particle
	vN_i = 17.6E-8 * 1/1000000	#(m3/s)		effective collision frequency ?
								#			of electrons in atmospheric gases
								#			per N !!!
	
	# effective collision frequency of ions in atmospheric gases
	v_i = vN_i * N				#(1/s)

	# electron concentration
	n_i = N * XH3O_a			#(1/m3)
	
	# electrical conductivity of positive ions represented by H3O+
	sigma2 = ( e_i*e_i * n_i ) / ( m_i * v_i )
	print('electrical conductivity H3O+:',sigma2, '(A2 * s3)/(kg * m3)')
	
	# total electrical conductivity
	sigma = sigma1 + sigma2
	print('electrical conductivity:',sigma, '(A2 * s3)/(kg * m3)') 
	
	y1.append(sigma)
	
	# temperature of the mixture (after equilibriation)
	Ta = mixture.T 	
	x1.append(Ta)
	
	T = T + dT
	
	# write calculated parameters to the csv file
	#csvwriter.writerow([Tb,Ta,XH3O_a,YH3O_a,Xe_a,Ye_a,n_e/1000000,n_e_mole,sigma1,sigma2,sigma])	
myfile.close()	
	
# Temperature of the gas after equilibration as a function 
# of the temperature before equilibration
fig1 = plt.figure(1)
rect1 = fig1.patch 
plot1 = fig1.add_subplot(1,1,1)
plot1.scatter(x1[1:],y1[1:])
plt.xlabel('T after equilibration [K]')
plt.ylabel('electrial conductivity [S/m]')
plt.savefig('temp_before_after.jpeg')

plt.show()



