#!/usr/bin/python3.4


"""
A freely-propagating, premixed hydrogen flat flame with multicomponent
transport properties.
"""

import cantera as ct
import numpy as np
import math

import csv
import matplotlib.pyplot as plt

#Integration module
import scipy.integrate as integ 
from scipy.integrate import trapz

#import mole fractions from measurement
from KG_premixed import n_frCH4_array
from KG_premixed import n_frO2_array
from KG_premixed import n_frN2_array
from KG_premixed import n_frCO2_array
#importoxygen content from measurement
from KG_premixed import ox_cont_array

# The main goal - calculated OH and CH signal
int_OH_rate_array = []
int_CH_rate_array = []
int_CO2_rate_array = []
int_CO2_rate_array_OLD = []

# For final charts
from KG_premixed import OH_signal_array 
from KG_premixed import CH_signal_array 
from KG_premixed import CO2_signal_array 

from KG_premixed import Marker #CO2 or N2
from KG_premixed import My_range #number of loops

for i in range(My_range):
	print(i, 'AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA')
	CH4 = str(n_frCH4_array[i])
	O2 = str(n_frO2_array[i])
	if Marker == 'N2':
		N2 = str(n_frN2_array[i])
	if Marker == 'CO2':
		CO2 = str(n_frCO2_array[i])
		
	MECHANISM = 'gri30LUMpremixed_ALL' # or gri30LUMpremixed_ALL

	# Simulation parameters
	p = 101325  # pressure [Pa]
	Tin = 300.0  # unburned gas temperature [K]
	if Marker == 'N2':
		reactants = 'CH4:'+ CH4 + ',' + 'O2:' + O2 + ',' + 'N2:' + N2  # premixed gas composition
	if Marker == 'CO2':
		reactants = 'CH4:'+ CH4 + ',' + 'O2:' + O2 + ',' + 'CO2:' + CO2  # premixed gas composition
	print(reactants)

	if MECHANISM == 'gri30LUMpremixed':
		if ox_cont_array[i] == 35:
			aa = 8
		if ox_cont_array[i] == 40:
			aa = 10
		if  ox_cont_array[i] == 45 or ox_cont_array[i] == 50 or ox_cont_array[i] == 55 or ox_cont_array[i] == 60:
			aa = 7 
	if MECHANISM == 'gri30LUMpremixed_ALL':
		if ox_cont_array[i] == 35:
			aa = 10
		if ox_cont_array[i] == 40:
			aa = 8
		if  ox_cont_array[i] == 45 or ox_cont_array[i] == 50 or ox_cont_array[i] == 55 or ox_cont_array[i] == 60:
			aa = 7 


	initial_grid = np.linspace(0.0, 0.03, aa)  # m ; number of points on the axis ; range 
	tol_ss = [1.0e-5, 1.0e-13]  # [rtol atol] for steady-state problem
	tol_ts = [1.0e-4, 1.0e-13]  # [rtol atol] for time stepping
	loglevel = 1  # amount of diagnostic output (0 to 8)
	refine_grid = True  # 'True' to enable refinement, 'False' to disable

	# IdealGasMix object used to compute mixture properties, set to the state of the
	# upstream fuel-air mixture
	
	
	
	gas = ct.Solution(MECHANISM + '.xml')
	gas.TPX = Tin, p, reactants

	# Flame object
	f = ct.FreeFlame(gas, initial_grid)
	f.flame.set_steady_tolerances(default=tol_ss)
	f.flame.set_transient_tolerances(default=tol_ts)
	
	f.show_solution()

	# Solve with the energy equation disabled
	f.energy_enabled = False
	f.transport_model = 'Mix'
	f.set_max_jac_age(10, 10)
	f.set_time_step(1e-5, [2, 5, 10, 20])
	f.solve(loglevel=loglevel, refine_grid=False)
	f.save('h2_adiabatic.xml', 'no_energy',
		   'solution with the energy equation disabled')

	# Solve with the energy equation enabled
	f.set_refine_criteria(ratio=3, slope=0.06, curve=0.12)
	f.energy_enabled = True
	f.solve(loglevel=loglevel, refine_grid=refine_grid)
	f.save('h2_adiabatic.xml', 'energy',
		   'solution with mixture-averaged transport')
	f.show_solution()
	print('mixture-averaged flamespeed = {0:7f} m/s'.format(f.u[0]))

	# Solve with multi-component transport properties
	f.transport_model = 'Multi'
	f.solve(loglevel, refine_grid)
	f.show_solution()
	print('multicomponent flamespeed = {0:7f} m/s'.format(f.u[0]))
	f.save('h2_adiabatic.xml','energy_multi',
		   'solution with multicomponent transport')

	# write the velocity, temperature, density, and mole fractions to a CSV file
	f.write_csv('h2_adiabatic.csv', quiet=False)

	#charts 
	#data:
	grid1 = f.grid
	rate347 = f.forward_rates_of_progress[346]
	rate348 = f.forward_rates_of_progress[347]
	OHex = f.concentrations[53]
	CHex = f.concentrations[54]
	
	#if MECHANISM == 'gri30LUMpremixed':
	CO2_signal = 3300 * 2.718 ** ( 2300/f.T  ) *  f.concentrations[14] * f.concentrations[2] 
	fig200 = plt.figure(200)
	plot1 = fig200.add_subplot(1,1,1) 
	plot1.plot(grid1,CO2_signal)
	plt.title('CO2 signal')
			
	#if MECHANISM == 'gri30LUMpremixed_ALL':
	rate362 = f.forward_rates_of_progress[361]
		
	fig2 = plt.figure(2)
	plot1 = fig2.add_subplot(1,1,1) 
	plot1.plot(grid1,rate362)
	plt.title('rate 362')

	#write to a file
	myfile = open('Oposed_check_results.csv','w')
	csvwriter = csv.writer(myfile)
	csvwriter.writerow(grid1)
	csvwriter.writerow(f.forward_rates_of_progress[347])
	csvwriter.writerow(f.concentrations[54])
	myfile.close()	

	fig1 = plt.figure(1)
	plot1 = fig1.add_subplot(2,2,1) 
	plot1.plot(grid1,rate347)
	plt.title('net rate R347')
	plt.xlim([0.01,0.015])
	plot2 = fig1.add_subplot(2,2,2) 
	plot2.plot(grid1,rate348)
	plt.title('net rate R348')
	plt.xlim([0.01,0.015])
	plot3 = fig1.add_subplot(2,2,3) 
	plot3.plot(grid1,CHex)
	plt.title('CHex')
	plt.xlim([0.01,0.015])
	plot4 = fig1.add_subplot(2,2,4) 
	plot4.plot(grid1,CHex)
	plt.title('OHex')
	plt.xlim([0.01,0.015])

	#plt.show()


	int_OH_rate = trapz(np.array(rate348),np.array(grid1))
	int_OH_rate_array.append(int_OH_rate)
	
	int_CH_rate = trapz(np.array(rate347),np.array(grid1))
	int_CH_rate_array.append(int_CH_rate)
	
	#if MECHANISM == 'gri30LUMpremixed':
	
	x_int_CO2 = grid1[:] #[7:182] -> 0.01:0.015
	
	int_CO2_signal = trapz(np.array(CO2_signal),x_int_CO2) 
	int_CO2_rate_array_OLD.append(int_CO2_signal)
	
	#if MECHANISM == 'gri30LUMpremixed_ALL':
		
	int_CO2_rate = trapz(np.array(rate362),np.array(grid1))
	int_CO2_rate_array.append(int_CO2_rate)

xlim = ( min(ox_cont_array) - 1 ),( max(ox_cont_array) + 1 )
'''
### Calculations ###

fig3 = plt.figure(4)
plot1 = fig3.add_subplot(3,1,1) 
plot1.scatter(ox_cont_array[0:(My_range)],int_OH_rate_array)
plt.xlabel('Oxygen content in the oxidizer')
plt.ylabel('OH signal')
plt.xlim(xlim)
plt.ylim([0,(max(int_OH_rate_array) + 0.1 * max(int_OH_rate_array))])

plot1 = fig3.add_subplot(3,1,2) 
plot1.scatter(ox_cont_array[0:(My_range)],int_CH_rate_array)
plt.xlabel('Oxygen content in the oxidizer')
plt.ylabel('CH signal')
plt.xlim(xlim)
plt.ylim([0,(max(int_CH_rate_array) + 0.1 * max(int_CH_rate_array))])

plot1 = fig3.add_subplot(3,1,3) 
plot1.scatter(ox_cont_array[0:(My_range)],int_CO2_rate_array)
plt.xlabel('Oxygen content in the oxidizer')
plt.ylabel('CO2 signal')
plt.xlim(xlim)
plt.ylim([0,(max(int_CO2_rate_array) + 0.1 * max(int_CO2_rate_array))])

plt.savefig('CALCULATIONS_' + Marker + '.jpeg')
'''
### FINAL ###

factor = 0.0001

# OH #

# adjust the trendline to measured points
rate_OH = max(OH_signal_array) / max(int_OH_rate_array)
int_OH_rate_array = [x*rate_OH*0.95 for x in int_OH_rate_array]
# lower the value by the factor 
OH_signal_array = [x*factor for x in OH_signal_array]
int_OH_rate_array = [x*factor for x in int_OH_rate_array]

fig5 = plt.figure(5)
plot1 = fig5.add_subplot(3,1,1) 
plot1.scatter(ox_cont_array[0:My_range],OH_signal_array, label = 'measurements')
plot2 = fig5.add_subplot(3,1,1) 
plot2.plot(ox_cont_array[0:(My_range)],int_OH_rate_array, color = '0.5')
plot21 = fig5.add_subplot(3,1,1) 
plot21.scatter(ox_cont_array[0:(My_range)],int_OH_rate_array, marker = 'h', color = '0.5', label = 'calculations')
plt.xlabel('Oxygen content in oxidizer')
#plt.legend(loc = '2')
plt.ylabel('OH signal')
plt.xlim(xlim)
plt.ylim([0,80])
#plt.ylim([0,(max(OH_signal_array) + 0.1 * max(OH_signal_array))])

# CH #

rate_CH = max(CH_signal_array) / max(int_CH_rate_array)
int_CH_rate_array = [x*rate_CH*0.95 for x in int_CH_rate_array]

CH_signal_array = [x*factor for x in CH_signal_array]
int_CH_rate_array = [x*factor for x in int_CH_rate_array]

plot3 = fig5.add_subplot(3,1,2) 
plot3.scatter(ox_cont_array[0:My_range],CH_signal_array, label = 'measurements')
plot4 = fig5.add_subplot(3,1,2) 
plot4.plot(ox_cont_array[0:(My_range)],int_CH_rate_array, color = '0.5')
plot41 = fig5.add_subplot(3,1,2) 
plot41.scatter(ox_cont_array[0:(My_range)],int_CH_rate_array, marker = 'h', color = '0.5')
plt.xlabel('Oxygen content in oxidizer')
plt.ylabel('CH signal')
plt.xlim(xlim)
plt.ylim([0,6])
#plt.ylim([0,(max(CH_signal_array) + 0.1 * max(CH_signal_array))])

# CO2 #

rate_CO2 = max(CO2_signal_array) / max(int_CO2_rate_array)
int_CO2_rate_array = [x*rate_CO2*1.05 for x in int_CO2_rate_array]

rate_CO2 = max(CO2_signal_array) / max(int_CO2_rate_array_OLD)
int_CO2_rate_array_OLD = [x*rate_CO2*1.05 for x in int_CO2_rate_array_OLD]

CO2_signal_array = [x*factor for x in CO2_signal_array]
int_CO2_rate_array = [x*factor for x in int_CO2_rate_array]
int_CO2_rate_array_OLD = [x*factor for x in int_CO2_rate_array_OLD]

plot5 = fig5.add_subplot(3,1,3) 
plot5.scatter(ox_cont_array[0:My_range],CO2_signal_array)
#################
plot6 = fig5.add_subplot(3,1,3) 
plot6.plot(ox_cont_array[0:(My_range)],int_CO2_rate_array, color = '0.5')
plot61 = fig5.add_subplot(3,1,3) 
plot61.plot(ox_cont_array[0:(My_range)],int_CO2_rate_array, marker = 'h', color = '0.5') #, label = 'mechanism')
#################
#plot6 = fig5.add_subplot(3,1,3) 
#plot6.plot(ox_cont_array[0:(My_range)],int_CO2_rate_array_OLD, color = '0.1')
#plot61 = fig5.add_subplot(3,1,3) 
#plot61.plot(ox_cont_array[0:(My_range)],int_CO2_rate_array_OLD, marker = 'x', color = '0.1', label = 'equation')
#################
plt.xlabel('Oxygen content in oxidizer')
plt.ylabel('CO2 signal')
plt.xlim(xlim)
#plt.ylim([0,(max(CO2_signal_array) + 0.1 * max(CO2_signal_array))])
plt.ylim([0,10])
plt.legend(loc = '2')
plt.savefig('FINAL_ALL' + Marker + '.jpeg')
print(int_CO2_rate_array_OLD)
plt.show()

#################### INTERPOLATION ############################

## Trend line ##
'''
# values from min to max oxygen content, step 1
x_tr_array = []
x_tr_start = min(ox_cont_array)
step = 1
for i in range( ((max(ox_cont_array)-min(ox_cont_array)) + 1 )):
	x_tr = x_tr_start + step * i
	x_tr_array.append(x_tr)
print(x_tr_array)

# Interpolation function
def interpolation(x,y,x1_int):
	coe = np.polyfit(x,y,3)
	loops = len(x1_int)
	y_cutoff = []
	for i in range(loops):
		y_fit_model = coe[0]*math.pow(x1_int[i],3) + coe[1]*math.pow(x1_int[i],2) + coe[2]*math.pow(x1_int[i],1) + coe[3]
		y_cutoff.append(y_fit_model)
	return(y_cutoff)
'''

# OH #
'''
# trendline based on calculated points
y_trendline_OH = interpolation(ox_cont_array[0:(My_range)],int_OH_rate_array,x_tr_array)

# adjust the trendline to measured points
rate_OH = max(OH_signal_array) / max(y_trendline_OH)
y_trendline_OH = [x*rate_OH*0.95 for x in y_trendline_OH]
# lower the value by the factor 
OH_signal_array = [x*factor for x in OH_signal_array]
y_trendline_OH = [x*factor for x in y_trendline_OH]
'''

# CH #
'''
y_trendline_CH = interpolation(ox_cont_array[0:(My_range)],int_CH_rate_array,x_tr_array)

rate_CH = max(CH_signal_array) / max(y_trendline_CH)
y_trendline_CH = [x*rate_CH*0.95 for x in y_trendline_CH]

CH_signal_array = [x*factor for x in CH_signal_array]
y_trendline_CH = [x*factor for x in y_trendline_CH]
'''

# CO2 #
'''
y_trendline_CO2 = interpolation(ox_cont_array[0:(My_range)],int_CO2_rate_array,x_tr_array)

rate_CO2 = max(CO2_signal_array) / max(y_trendline_CO2)
y_trendline_CO2 = [x*rate_CO2*1.05 for x in y_trendline_CO2]

CO2_signal_array = [x*factor for x in CO2_signal_array]
y_trendline_CO2 = [x*factor for x in y_trendline_CO2]
'''

